* {
  margin: 0;
  padding: 0;
  font-family: 'Open Sans';
}

body,
html,
canvas {
  overflow: hidden;
  height: 100%;
}

body,
html {
  background: #111;
  touch-action: none;
}

a {
  text-decoration: none;
  color: #eab755;
  display: inline-block;
}

select {
  border-radius: 0;
  -webkit-appearance: none;
  -moz-appearance: none;
  appearance: none;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
  background: transparent;
  -webkit-user-select: none;
  -moz-user-select: -moz-none;
  -ms-user-select: none;
  user-select: none;
  cursor: pointer;
  border: none;
  color: #333131;
  font-size: 13px;
  padding: 3px 6px;
  border-radius: 5px;
  background-color: #c3c3c3;
  background-image: url(../images/icons/caret-icon.png);
  background-size: 20px;
  background-repeat: no-repeat;
  background-position: 95%;
  min-width: 120px;
  max-width: 200px;
  transition: opacity 0.3s;
}

select::-ms-expand {
  display: none;
}

select:disabled {
  cursor: default;
  opacity: 0.4;
}

.no-highlight {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -khtml-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

/**
* Canvases
**/

#canvas-container {
  position: absolute;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  overflow: hidden;
}

#canvas-target {
  position: absolute;
  top: -60px;
  right: -90px;
  bottom: -90px;
  left: -90px;
}

#lod-canvas {
  position: absolute;
  top: 60px;
  left: 100px;
  z-index: 1;
  background: #000;
  max-height: 90%;
}

#character-canvas {
  position: absolute;
  top: 60px;
  left: 100px;
  z-index: 1;
}

#pixplot-canvas {
  height: 100%;
  width: 100%;
}

#pixplot-canvas.pan {
  cursor: grab;
}

#pixplot-canvas.pan:active {
  cursor: grabbing;
}

#pixplot-canvas.select {
  cursor: crosshair;
}

#stats {
  position: absolute;
  top: 60px;
  left: 106px;
  width: 80px;
}

/**
* Header
**/

#header {
  height: 60px;
  width: 100%;
  position: fixed;
  top: 0px;
  overflow: hidden;
  box-shadow: 2px 1px 2px rgba(39, 39, 39, 0.2);
  font-size: 0;
}

#header,
#logo {
  background: #333;
  position: relative;
  z-index: 4;
}

#logo {
  width: 40px;
  padding: 10px;
  height: 40px;
}

#app-name {
  display: inline-block;
  height: 100%;
  vertical-align: top;
  padding: 18px 34px;
  box-sizing: border-box;
  font-weight: 300;
  background: #444;
  color: #fff;
  font-size: 21px;
  letter-spacing: 0.1em;
  position: relative;
  z-index: 1;
}

.tagline {
  color: #999;
  font-size: 16px;
  padding: 20px 25px;
  font-weight: 400;
  letter-spacing: .025em;
  display: inline-block;
  vertical-align: top;
  height: 100%;
  box-sizing: border-box;
  background: inherit;
  position: relative;
  z-index: 1;
}

@media(max-width: 335px) {
  #header #logo {
    display: none;
  }

  #header .app-name {
    width: 100%;
    text-align: center;
    padding: 18px 0;
  }
}

/**
* Header Controls
**/

#header-controls {
  display: inline-block;
  position: absolute;
  right: 20px;
  top: 0;
  left: 515px;
  padding: 15px 0;
  min-width: 280px;
  opacity: 0;
  transition: opacity 0.5s;
  text-align: right;
  white-space: nowrap;
}

#header-controls * {
  display: inline-block;
}

/**
* Image search
**/

#image-search-container {
  margin-right: 20px;
  position: relative;
  top: -1px;
  display: none;
}

#image-search {
  padding-right: 30px;
  font-size: 13px;
}

#image-search-icon {
  width: 14px;
  position: absolute;
  right: 6px;
  top: 6px;
  transform: scaleX(-1);
}

/**
* Filters
**/

#filters {
  vertical-align: top;
  position: relative;
  top: 2px;
  display: inline-block;
  margin: 0 10px;
}

/**
* Range slider
**/

input[type=range] {
  -webkit-appearance: none;
  appearance: none;
  width: 100%;
  height: 2px;
  border-radius: 10px;
  background: #c3c3c3;
  outline: none;
}

input[type=range]::-webkit-slider-thumb {
  -webkit-appearance: none;
  appearance: none;
  width: 14px;
  height: 14px;
  background: #eee;
  cursor: pointer;
  border-radius: 14px;
}

input[type=range]::-moz-range-thumb {
  -webkit-appearance: none;
  appearance: none;
  width: 14px;
  height: 14px;
  background: #eee;
  cursor: pointer;
  border-radius: 14px;
}

input[type=range]::-ms-thumb {
  -webkit-appearance: none;
  appearance: none;
  width: 14px;
  height: 14px;
  background: #eee;
  cursor: pointer;
  border-radius: 14px;
}

#range-slider {
  margin-right: 10px;
  width: 100px;
  display: inline-block;
}

#range-slider label {
  color: #c3c3c3;
  font-size: 9px;
  letter-spacing: 0.05em;
  text-transform: uppercase;
  display: block;
}

#pointsize-range-input {
  margin-top: 6px;
}

/**
* Dual range slider
**/

#date-slider {
  height: 6px;
  width: 100px;
  display: inline-block;
  vertical-align: top;
  position: relative;
  top: 19px;
  margin-left: 10px;
}

#date-slider .noUi-connect {
  background: #eab755;
}

#date-slider .noUi-handle {
  background: #eee;
  box-shadow: none;
  width: 16px;
  height: 16px;
  border-radius: 16px;
  right: -8px;
  border: 1px solid #888;
}

#date-slider .noUi-handle::before,
#date-slider .noUi-handle::after {
  display: none;
}

#date-slider .noUi-tooltip {
  font-size: 9px;
  padding: 0;
  margin-bottom: 1px;
  border: 1px solid transparent;
  background: transparent;
  color: #c4c4c4;
}

/**
* Icons
**/

#icons {
  display: inline-block;
  vertical-align: top;
  margin-top: 5px;
}

#icons img {
  height: 19px;
  margin: 0 8px;
  opacity: 0.2;
  cursor: pointer;
  vertical-align: top;
}

#icons img.active {
  opacity: 1;
}

#layout-date,
#layout-categorical {
  display: none;
}

/**
* Jitter
**/

#jitter-container {
  position: absolute;
  top: 77px;
  right: 20px;
  z-index: 1;
  background: #444;
  font-size: 12px;
  padding: 5px;
  border-radius: 3px;
  color: #dedede;
  font-family: courier;
  text-transform: uppercase;
  transition: opacity 0.5s;
  opacity: 0;
}

#jitter-container.visible {
  opacity: 1;
}

#jitter-container.disabled {
  opacity: 0.5;
  pointer-events: none;
  cursor: not-allowed;
}

#jitter-container,
#jitter-container * {
  cursor: pointer;
}

#jitter-container input {
  position: relative;
  top: 2px;
}

/**
* Nav
**/

nav {
  position: absolute;
  z-index: 1;
  top: 0;
  left: 0;
  bottom: 0;
  width: 106px;
  height: auto;
  box-sizing: border-box;
  padding: 60px 0 0;
  background: rgba(17,17,17,0.95);
  box-shadow: 1px 0px 2px rgba(0, 0, 0, 0.7);
  transition: opacity 0.5s;
  color: #ddd;
  opacity: 1;
  cursor: default;
  overflow: hidden;
}

#nav-inner {
  height: 100%;
  max-height: 100%;
  overflow-x: hidden;
  overflow-y: auto;
  padding-right: 30px;
  background: #222;
  transition: opacity 0.5s;
  box-sizing: content-box;
  width: 100%;
}

nav.disabled #nav-inner {
  opacity: 0.3;
  cursor: not-allowed;
  pointer-events: none;
}

nav h2 {
  font-size: 0.9em;
  line-height: 1.05em;
  text-align: center;
  margin: 15px 0 7px;
  text-transform: uppercase;
  font-weight: 400;
}

#hotspots {
  width: 75px;
  padding: 0 15px;
}

.hotspot {
  padding: 6px 0;
  display: inline-block;
  text-align: center;
  width: 75px;
  font-size: 13px;
  position: relative;
  transition: opacity 0.3s;
}

.hotspot-image {
  width: 100%;
  cursor: pointer;
  display: block;
}

.hotspot-bar-container {
  height: 5px;
  background: #444;
  margin: 4px 0;
}

.hotspot-bar-inner {
  transition: width 1s;
  background: #eab755;
  height: 100%;
  width: 0;
}

.hotspot .hotspot-label {
  cursor: text;
}

.hotspot-action {
  position: absolute;
  top: 1px;
  width: 12px;
  height: 12px;
  vertical-align: top;
  box-shadow: 0px 0px 3px #000;
  cursor: pointer;
  padding: 1px;
  font-size: 8px;
  line-height: 12px;
  border-radius: 2px;
}

.hotspot .remove-hotspot-x {
  left: -7px;
  background: #9c1717;
  color: #eee;
}

.hotspot .remove-hotspot-x:hover {
  background: firebrick;
}

.hotspot .drag-hotspot,
.hotspot .refresh-hotspot {
  background: #444;
}

.hotspot .drag-hotspot {
  right: -7px;
  background: #4c4c4c;
  color: #fff;
  font-size: 11px;
  line-height: 12px;
  cursor: grab;
}

.hotspot .drag-hotspot:hover,
.hotspot .refresh-hotspot:hover {
  background: #666;
}

.hotspot.dragging,
.hotspot.dragging .drag-hotspot {
  cursor: grabbing;
}

.hotspot .refresh-hotspot {
  position: absolute;
  right: -7px;
  top: 20px;
  width: 10px;
  height: 10px;
  padding: 2px;
}

/**
* Hotspot actions
**/

#hotspot-actions {
  position: absolute;
  left: 115px;
  bottom: 10px;
  width: 130px;
}

#create-hotspot {
  color: #ccc;
  border: 2px solid #ccc;
  display: none;
  margin-top: 8px;
}

#create-hotspot:hover {
  background: #ccc;
  color: #000;
}

#save-hotspots {
  white-space: nowrap;
  background: #eab755;
  color: #000;
  border: 2px solid #eab755;
  display: none;
}

/**
* New hotspot button in hotspot column
**/

#new-hotspot {
  height: 90px;
  text-align: center;
  font-size: 45px;
  line-height: 90px;
  cursor: pointer;
  margin-top: 7px;
  font-weight: 100;
  color: #dcdcdc;
  position: relative;
}

#new-hotspot::after {
  content: '+';
}

#new-hotspot div {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  background: #ababab;
  z-index: 0;
  margin: 1px;
  opacity: 0.2;
}

#new-hotspot:hover {
  outline-color: #eab755;
  color: #fff;
}

.marching-ants {
  background-size: 6px 1px, 6px 1px, 1px 6px, 1px 6px;
  background-position: 0 0, 0 100%, 0 0, 100% 0;
  background-repeat: repeat-x, repeat-x, repeat-y, repeat-y;
  -webkit-animation: marching-ants 3s;
          animation: marching-ants 3s;
  -webkit-animation-timing-function: linear;
          animation-timing-function: linear;
  -webkit-animation-iteration-count: infinite;
          animation-iteration-count: infinite;
  -webkit-animation-play-state: running;
          animation-play-state: running;
  background-image: -webkit-gradient(linear, left top, right top, color-stop(50%, #e5e5e5), color-stop(50%, #222)),
                    -webkit-gradient(linear, left top, right top, color-stop(50%, #e5e5e5), color-stop(50%, #222)),
                    -webkit-gradient(linear, left top, left bottom, color-stop(50%, #e5e5e5), color-stop(50%, #222)),
                    -webkit-gradient(linear, left top, left bottom, color-stop(50%, #e5e5e5), color-stop(50%, #222));
  background-image: linear-gradient(to right, #e5e5e5 50%, #222 50%),
                    linear-gradient(to right, #e5e5e5 50%, #222 50%),
                    linear-gradient(to bottom, #e5e5e5 50%, #222 50%),
                    linear-gradient(to bottom, #e5e5e5 50%, #222 50%);
  color: #fff;
}

@-webkit-keyframes marching-ants {
  0% { background-position: 0 0,  0 100%,  0 0,  100% 0; }
  100% { background-position: 40px 0, -40px 100%, 0 -40px, 100% 40px; }
}

@keyframes marching-ants {
  0% { background-position: 0 0,  0 100%,  0 0,  100% 0; }
  100% { background-position: 40px 0, -40px 100%, 0 -40px, 100% 40px; }
}

/**
* New hotspot button in the plot
**/

#new-hotspot-icon {
  position: absolute;
  top: 0;
  left: 0;
  font-size: 10px;
  color: #fff;
  background: #333;
  padding: 4px 8px;
  text-align: center;
  border: 1px solid #444;
  transition: transform 0.3s;
  border-radius: 3px;
  cursor: pointer;
}

/**
* Loader
**/

#loader-scene {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  color: #fff;
  padding: 40px;
  padding-top: 120px;
  transition: transform 1s;
  z-index: 3;
  background: #000;
  max-height: 100%;
  width: 100%;
  box-sizing: border-box;
  overflow-x: hidden;
  overflow-y: auto;
  display: flex;
  flex-direction: column;
  justify-content: center;
}

#loader-scene.hidden {
  transform: translateY(-100%);
}

#loader-text {
  margin: 10px -50px;
  text-align: center;
}

.welcome {
  max-width: 700px;
  margin: 0 auto;
}

@keyframes exit {
  0% {
    transform: translateX(0);
    animation-iteration-count: 1;
  }
  1% {
    transform: translateX(500vw);
    animation-iteration-count: 1;
  }
}


/**
* Loader icon
**/

.loader-container {
  max-width: 70px;
  margin: 30px auto;
  text-align: center;
}

.loader-icon {
  width: 70px;
  height: 70px;
}

.block {
  position: relative;
  height: 20px;
  width: 20px;
  display: inline-block;
  background: #eab755;
  transition: all 0.8s;
  animation: rot 5s linear infinite;
  animation-fill-mode: forwards;
}

.block:nth-child(1) {
  animation-delay: 3s;
}

.block:nth-child(2) {
  animation-delay: 1.5s;
  animation: rot 15s linear infinite;
}

.block:nth-child(3) {
  animation-delay: 2s;
}

.block:nth-child(4) {
  animation-delay: 0.2s;
}

.block:nth-child(5) {
  animation-delay: 4s;
}

.block:nth-child(6) {
  animation-delay: 2s;
  animation: rot 7s linear infinite;
}

.block:nth-child(7) {
  animation-delay: 0.4s;
}

.block:nth-child(8) {
  animation-delay: 1.5s;
  animation: rot 6s linear infinite;
}

.block:nth-child(9) {
  animation-delay: 25s;
  animation: rot 8s linear infinite;
}

@keyframes rot {
  0% {
    transform: none;
  }
  20% {
    transform: rotateZ(-90deg) rotateY(180deg);
  }
  40% {
    background: chocolate;
    transform: none;
  }
  60% {
    background: white;
  }
  80% {
    background: cornflowerblue;
  }
  90% {
    transform: none;
    background: #222;
  }
}

/**
* Enter button
**/

button {
  border: none;
  background: #eab755;
  padding: 10px 40px;
  display: block;
  margin: 0 auto;
  font-size: 14px;
  font-weight: 600;
  opacity: 0.4;
  transition: opacity .3s;
  cursor: pointer;
}

button.active {
  opacity: 1;
}

/**
* Browser messages
**/

#webgl-not-available {
  text-align: center;
  display: none;
}

noscript {
  text-align: center;
  display: block;
  width: 100%;
}

.browser-message {
  position: relative;
  top: 90px;
  z-index: 2;
  background: #c62d1e;
  color: #fff;
  padding: 12px 17px;
  font-size: 17px;
}

/**
* Shared modal styles
**/

.modal-content {
  height: 100%;
  width: 100%;
  display: flex;
  flex-direction: column;
  position: relative;
}

.modal-x {
  color: #ccc;
  position: absolute;
  top: 20px;
  right: 20px;
  font-size: 22px;
  cursor: pointer;
  padding: 0;
}

.modal-top {
  padding: 60px 10%;
  flex: 1;
  box-sizing: border-box;
  flex-direction: column;
  justify-content: center;
  display: flex;
  align-items: center;
  height: calc(100vh - 175px);
}

.modal-top h2 {
  font-weight: 100;
  color: #666;
  text-align: center;
  font-size: 25px;
}

.modal-top p {
  font-size: 14px;
  margin: 10px 0 20px;
  text-align: center;
}

.modal-top-content {
  padding: 40px;
  background: #eee;
  box-sizing: border-box;
}

.modal-bottom {
  background: #eee;
  padding: 20px 10%;
  font-size: 12px;
  box-sizing: border-box;
  height: 115px;
}

.modal-bottom-content {
  font-size: 12px;
  line-height: 18px;
  display: flex;
  flex-direction: row;
  justify-content: space-between;
}

.modal-bottom-content b {
  margin-bottom: 5px;
  display: inline-block;
  font-size: 15px;
}

/**
* Selected Image modal
**/

#selected-image-target {
  position: absolute;
  top: 60px;
  right: 0;
  bottom: 0;
  left: 0;
  background: rgba(17, 17, 17, 0.98);
  z-index: 10;
  display: none;
}

#selected-image,
#previous-image {
  max-height: 100%;
  max-width: 100%;
  box-sizing: border-box;
  background-color: #222;
}

#previous-image {
  display: none;
}

#selected-image-target .modal-content {
  justify-content: center;
}

#selected-image-target .modal-top-content {
  background: transparent;
  height: 100%;
  position: relative;
  padding: 0;
  text-align: center;
  display: flex;
  flex-direction: column;
  justify-content: center;
  align-items: center;
}

#selected-image {
  padding: 25px;
  background-color: #333;
}

.image-caret {
  position: absolute;
  height: 80px;
  background-color: transparent;
  top: 50%;
  margin-top: -40px;
  cursor: pointer;
  opacity: 0.6;
}

#caret-left {
  left: -150px;
}

#caret-right {
  right: -150px;
  transform: rotate(180deg);
}

/**
* Selected Image Actions
**/

.icons {
  display: inline-block;
  width: 40px;
  position: absolute;
  bottom: 25px;
  right: -50px;
  white-space: initial;
}

.icons a {
  height: 32px;
  width: 32px;
  background: #cecbcb;
  padding: 6px;
  box-sizing: border-box;
  border-radius: 4px;
  cursor: pointer;
  margin: 3px;
}

.icons img {
  width: 100%;
}

/**
* Selected Image Meta
**/

#selected-image-meta {
  text-align: left;
  color: #555;
  height: 115px;
  z-index: 1;
}

.meta-content {
  padding: 24px 40px;
  transition: transform 0.3s;
}

#image-title {
  font-weight: 600;
  margin-bottom: 6px;
  font-size: 16px;
}

#image-text {
  font-size: 13px;
  font-weight: 400;
  max-height: 73px;
  overflow: auto;
}

#image-date {
  margin-left: 4px;
}

.modal-text-left {
  flex: 0.5;
  display: inline-block;
  padding-right: 50px;
  box-sizing: border-box;
}

.modal-text-right {
  flex: 0.5;
  display: inline-block;
  vertical-align: top;
}

#meta-category-container {
  margin: 0 -2px 10px;
}

.meta-category {
  display: inline-block;
  padding: 4px 6px;
  margin: 2px;
  background: #d3d3d3;
  color: #777;
  border-radius: 5px;
  font-size: 13px;
  font-weight: 600;
}

@media (max-width: 800px) {
  .modal-text-left {
    width: 100%;
  }

  .modal-text-right {
    width: 100%;
  }
}

/**
* Multi-image Selection
**/

#selection-icons {
  position: absolute;
  bottom: 10px;
  right: 12px;
  display: inline-block;
  width: 30px;
  padding: 0 5px;
  background: #333;
  border-radius: 4px;
}

#selection-icons img {
  display: block;
  padding: 7px;
  background: lightgray;
  border-radius: 6px;
  max-width: 100%;
  box-sizing: border-box;
  margin: 5px 0;
  opacity: 0.4;
  cursor: pointer;
}

#selection-icons img.active {
  opacity: 1.0;
}

#selection-icons img:hover {
  background: #eab755;
  opacity: 1.0;
}

#select-tooltip {
  position: absolute;
  left: -168px;
  z-index: 1;
  color: #fff;
  bottom: 0px;
  width: 126px;
  font-size: 12px;
  padding: 13px;
  border: 1px solid #717171;
  display: none;
  border-radius: 3px;
}

#select-tooltip::before {
  content: '';
  width: 10px;
  height: 10px;
  transform: rotate(45deg);
  position: absolute;
  right: -6px;
  bottom: 14px;
  border-right: 1px solid #717171;
  border-top: 1px solid #717171;
}

#select-tooltip,
#select-tooltip::before {
  background: #222;
}

#select-tooltip-heading {
  font-weight: 800;
}

#select-tooltip p {
  line-height: 1.4em;
  margin: 5px 0;
}

#select-tooltip-button {
  display: inline-block;
  border: 2px solid #F2C54A;
  padding: 3px 20px;
  margin-top: 4px;
  border-radius: 3px;
  color: #F2C54A;
  font-weight: 800;
  cursor: pointer;
}

#select-tooltip-button:hover {
  background: #F2C54A;
  color: #222;
}

/**
* Multi-image Modal
**/

#view-selected-container {
  display: none;
  position: absolute;
  bottom: 10px;
  right: 60px;
  z-index: 1;
}

.button {
  font-size: 12px;
  padding: 3px 10px;
  border-radius: 4px;
  font-weight: 600;
  letter-spacing: 0.02em;
  text-align: center;
  cursor: pointer;
  transition: opacity 0.5s;
  white-space: nowrap;
}

.hotspot-button {
  width: 106px;
  text-align: center;
}

#view-selected {
  background: #222;
  border: 2px solid #eab755;
  color: #eab755;
}

#view-selected:hover {
  background: #eab755;
  color: #222;
}

#selected-images-count {
  color: #fff;
  font-size: 10px;
  white-space: nowrap;
  padding: 10px;
  background: #222;
  border-radius: 3px;
  text-align: center;
  margin-bottom: 5px;
  border: 1px solid #717171;
}

#count-target {
  color: #eab755;
  font-weight: 800;
  text-align: center;
  font-size: 20px;
}

/**
* Selected Image Modal
**/

#selected-images-modal {
  position: absolute;
  top: 60px;
  left: 0;
  right: 0;
  bottom: 0;
  z-index: 2;
  background: #111;
  overflow: auto;
  display: none;
}

#selected-images-grid {
  display: flex;
  flex-wrap: wrap;
}

#selected-images-target {
  max-height: 100%;
  overflow: auto;
}

#selected-images-x {
  position: absolute;
  top: 5px;
  right: 8px;
  color: #ccc;
  cursor: pointer;
}

.selected-image {
  width: 90px;
  height: 90px;
  display: inline-block;
  margin: 9px;
  display: flex;
  justify-content: center;
  align-items: center;
  position: relative;
  border: 0.125em solid #707070;
  background: #222;
}

.selected-image .background-image {
  width: 100%;
  height: 100%;
  background-size: cover;
  background-repeat: no-repeat;
  background-position: center top;
  padding: 0.125px;
  box-sizing: border-box;
}

.selected-image .background-image.unselected {
  opacity: 0.1;
}

#selected-images-grid:hover .toggle-selection {
  opacity: 1;
}

.toggle-selection {
  position: absolute;
  z-index: 10;
  color: #fff;
  background: #000;
  width: 18px;
  height: 18px;
  top: 5px;
  right: 5px;
  text-align: center;
  border: none;
  outline: none;
  font-size: 10px;
  line-height: 18px;
  opacity: 0;
  transition: opacity 0.3s;
  cursor: pointer;
}

input[type=text] {
  padding: 4px 10px;
  font-size: 15px;
  border-radius: 3px;
  border: none;
  min-width: 120px;
  background: #ddd;
}

#download-link {
  vertical-align: top;
}

#download-link button {
  display: inline-block;
  margin-left: -5px;
  padding: 0.733em 20px;
  border-top-right-radius: 4px;
  border-bottom-right-radius: 4px;
  vertical-align: top;
  height: 40px;
}

#selected-images-modal input {
  border-top-left-radius: 4px;
  border-bottom-left-radius: 4px;
  height: 40px;
  box-sizing: border-box;
}

#selected-images-modal .modal-text-right {
  white-space: nowrap;
}

#download-as {
  margin-top: 10px;
}

#download-as span {
  margin-right: 2px;
}

#download-as button {
  opacity: 1;
  background-color: #d2d2d2;
  padding: 4px 8px;
  display: inline-block;
  border-radius: 4px;
  font-size: 12px;
  margin: 2px;
}

#download-as button.active {
  background-color: #eab755;
}

/**
* Tooltip
**/

#tooltip {
  background: lightgray;
  z-index: 100;
  padding: 3px 8px;
  font-size: 12px;
  white-space: nowrap;
  border-radius: 3px;
  color: #333;
  opacity: 0.97;
}

#tooltip::after {
  content: '';
  position: absolute;
  top: -4px;
  width: 8px;
  height: 8px;
  margin-left: -4px;
  background: inherit;
  right: 15px;
  transform: rotate(45deg);
  z-index: -1;
}